@@ -33,7 +33,9 @@ from gettext import gettext as _
 import gettext
 import os
 import subprocess
+import multiprocessing
 import logging
+import traceback
 import threading
 import sys
 import time
@@ -106,6 +108,29 @@ POCKET_SECURITY = "security"
 POCKET_UPDATES = "updates"
 POCKET_BACKPORTS = "backports"
 
+SIGTERM = 15
+
+# Overload multiprocessing.Process.run() so we can catch run's exception.
+# https://stackoverflow.com/a/63773140
+class Process(multiprocessing.Process):
+    def __init__(self, *args, **kwargs):
+        multiprocessing.Process.__init__(self, *args, **kwargs)
+        self._pconn, self._cconn = multiprocessing.Pipe()
+        self._exception = None
+    def run(self):
+        try:
+            multiprocessing.Process.run(self)
+            self._cconn.send(None)
+        except Exception:
+            tb = traceback.format_exc()
+            self._cconn.send(tb)
+    @property
+    def exception(self):
+        if self._pconn.poll():
+            self._exception = self._pconn.recv()
+        if not (self._exception is None):
+            return Exception(self._exception)
+
 
 class UpdateSubscriptions(object):
     security = False
@@ -218,8 +243,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             "KeysModified", self.on_keys_modified)
         self.backend.connect_to_signal(
             "AuthFailed", self.on_auth_failed)
-        self.backend.connect_to_signal(
-            "CdromScanFailed", self.on_cdrom_scan_failed)
 
         # Reload dbus backend, for if there are any sources changed.
         self.backend.Reload();
@@ -234,6 +257,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.init_apt_cache()
 
         self.pk_task = None
+        self.search_process = None
 
         # Put some life into the user interface:
         self.init_auto_update()
@@ -245,7 +269,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # Setup the ISV sources list
         self.init_isv_sources()
         self.show_isv_sources()
-        self.show_cdrom_sources()
         # Setup and show the distro elements
         self.init_distro()
         self.show_distro()
@@ -274,15 +297,25 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # Make sure window is as small as possible initially.
         self.window_main.resize(1, 1)
 
+    def do_shutdown(self):
+        if self.search_process and self.search_process.is_alive():
+            self.search_process.terminate()
+        SimpleGtkbuilderApp.do_shutdown(self)
+
     def on_main_notebook_page_switched(self, notebook, page, page_num):
         # On the additional drivers page, don't show the backend revert button.
-        if page == self.vbox_drivers:
+        if page == self.vbox_drivers or page == self.vbox_auth:
             self.button_revert.set_visible(False)
-            if not self.detect_called:
-                GLib.idle_add(lambda: threading.Thread(target=self.detect_drivers).start())
+            self.maybe_detect_drivers()
         else:
             self.button_revert.set_visible(True)
 
+    def maybe_detect_drivers(self):
+        if not self.search_process or not self.search_process.is_alive():
+            t = threading.Thread(target=self.detect_drivers)
+            self.label_driver_detail.set_text(_("Searching for available drivers..."))
+            t.start()
+
     def update_interface(self):
         """ abstract interface to keep the UI alive """
         while Gtk.events_pending():
@@ -551,14 +584,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.checkbutton_source_code.set_active(False)
             self.checkbutton_source_code.set_inconsistent(False)
 
-        # Will show a short explanation if no CDROMs are used
-        if len(self.get_cdrom_sources()) == 0:
-            self.scrolledwindow_cd.hide()
-            self.scrolledwindow_no_cd.show()
-        else:
-            self.scrolledwindow_cd.show()
-            self.scrolledwindow_no_cd.hide()
-
         # provide a list of mirrors
         server_store = self.combobox_server.get_model()
         server_store.clear()
@@ -594,9 +619,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             print("CHILD SOURCES")
             for source in self.distro.child_sources:
                 self.print_source_entry(source)
-            print("CDROM SOURCES")
-            for source in self.distro.cdrom_sources:
-                self.print_source_entry(source)
             print("SOURCE CODE SOURCES")
             for source in self.distro.source_code_sources:
                 self.print_source_entry(source)
@@ -814,12 +836,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # STORE_SOURCE - the source entry object
         # STORE_SEPARATOR - if the entry is a separator
         # STORE_VISIBLE - if the entry is shown or hidden
-        self.cdrom_store = Gtk.ListStore(GObject.TYPE_BOOLEAN,
-                                         GObject.TYPE_STRING,
-                                         GObject.TYPE_PYOBJECT,
-                                         GObject.TYPE_BOOLEAN,
-                                         GObject.TYPE_BOOLEAN)
-        self.treeview_cdroms.set_model(self.cdrom_store)
         self.source_store = Gtk.ListStore(GObject.TYPE_BOOLEAN,
                                           GObject.TYPE_STRING,
                                           GObject.TYPE_PYOBJECT,
@@ -840,13 +856,10 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         cell_toggle.set_property("xpad", 2)
         cell_toggle.set_property("ypad", 2)
         self.handlers[cell_toggle] = cell_toggle.connect('toggled',
-                                                  self.on_isv_source_toggled,
-                                                  self.cdrom_store)
+                                                  self.on_isv_source_toggled)
         col_active = Gtk.TreeViewColumn(_("Active"), cell_toggle,
                                         active=COLUMN_ACTIVE)
 
-        self.treeview_cdroms.append_column(col_active)
-        self.treeview_cdroms.append_column(col_desc)
 
         cell_desc = Gtk.CellRendererText()
         cell_desc.set_property("xpad", 2)
@@ -906,7 +919,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.keys_store = Gtk.ListStore(str)
         self.treeview_auth.set_model(self.keys_store)
         tr = Gtk.CellRendererText()
-        keys_col = Gtk.TreeViewColumn(_("Key"), tr, text=0)
+        keys_col = Gtk.TreeViewColumn(_("Key"), tr, markup=0)
         self.treeview_auth.append_column(keys_col)
         try:
             self.treeview_auth.enable_model_drag_dest(
@@ -982,7 +995,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.reload_sourceslist()
         self.show_distro()
         self.show_isv_sources()
-        self.show_cdrom_sources()
         self.button_revert.set_sensitive(True)
         self.modified_sourceslist = True
 
@@ -993,11 +1005,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.on_config_modified()
         self.on_keys_modified()
 
-    def on_cdrom_scan_failed(self):
-        error(self.window_main,
-              _("Error scanning the CD"),
-              _("Could not find a suitable CD."))
-
     # helpers
     def show_isv_sources(self):
         """ Show the repositories of independent software vendors in the
@@ -1014,15 +1021,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.button_remove.set_sensitive(False)
             self.button_edit.set_sensitive(False)
 
-    def show_cdrom_sources(self):
-        """ Show CD-ROM/DVD based repositories of the currently used distro in
-            the CDROM based sources list """
-        self.cdrom_store.clear()
-        for source in self.get_cdrom_sources():
-            contents = self.render_source(source)
-            self.cdrom_store.append([not source.disabled, contents,
-                                    source, False, True])
-
     def is_separator(self, model, iter, column):
         """ Return true if the selected row is a separator """
         try:
@@ -1033,8 +1031,13 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
     def show_keys(self):
         self.keys_store.clear()
-        for key in self.apt_key.list():
-            self.keys_store.append([key])
+        for keyring in self.apt_key.list():
+            keyring = GLib.markup_escape_text(keyring)
+            filename, details = keyring.split("\n", 1)
+            if details:
+                self.keys_store.append([f"<b>{filename}</b>\n{details}"])
+            else:
+                self.keys_store.append([f"<b>{filename}</b>\nNo details"])
 
     def on_combobox_update_interval_changed(self, widget):
         """Set the update automation interval to the chosen one"""
@@ -1107,12 +1110,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             return
         # allow to remove the selected channel
         self.button_remove.set_sensitive(True)
-        # disable editing of cdrom sources
-        source_entry = model.get_value(iter, LIST_ENTRY_OBJ)
-        if source_entry.uri.startswith("cdrom:"):
-            self.button_edit.set_sensitive(False)
-        else:
-            self.button_edit.set_sensitive(True)
+        self.button_edit.set_sensitive(True)
 
     def on_remove_clicked(self, widget):
         """Remove the selected source"""
@@ -1157,21 +1155,14 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             return
         key = model.get_value(a_iter, 0)
         try:
-            if not self.backend.RemoveKey(key[:16]):
-                error(self.main,
+            if not self.backend.RemoveKey(key.split("\n", 1)[0].replace("<b>", "").replace("</b>", "")):
+                error(self.window_main,
                       _("Error removing the key"),
                       _("The key you selected could not be removed. "
                         "Please report this as a bug."))
         except dbus.DBusException as e:
             maybe_log_authentication_canceled_error(e)
 
-    def on_restore_clicked(self, widget):
-        """Restore the original keys"""
-        try:
-            self.backend.UpdateKeys()
-        except dbus.DBusException as e:
-            maybe_log_authentication_canceled_error(e)
-
     def on_delete_event(self, widget, args):
         """Close the window if requested"""
         self.on_close_button(widget)
@@ -1188,13 +1179,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
         self.quit()
 
-    def on_button_add_cdrom_clicked(self, widget):
-        """ when a cdrom is requested for adding """
-        try:
-            self.backend.AddCdromSource()
-        except dbus.DBusException as e:
-            maybe_log_authentication_canceled_error(e)
-
     def on_driver_changes_progress(self, progress, ptype, data=None):
         #print(progress)
         self.button_driver_revert.set_visible(False)
@@ -1314,9 +1298,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         elif (desktop == 'XFCE' and
               os.path.exists('/usr/bin/xfce4-session-logout')):
             subprocess.call(['xfce4-session-logout'])
-        elif (desktop == 'LXDE' and
-              os.path.exists('/usr/bin/lubuntu-logout')):
-            subprocess.call(['lubuntu-logout'])
         elif (desktop == 'LXQt' and
               os.path.exists('/usr/bin/lxqt-leave')):
             subprocess.call(['lxqt-leave'])
@@ -1347,18 +1328,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.box_driver_action.pack_end(self.button_driver_restart, False, False, 0)
         self.box_driver_action.pack_end(self.button_driver_cancel, False, False, 0)
 
-        props = {"halign":Gtk.Align.CENTER, "valign":Gtk.Align.CENTER,
-                 "vexpand":True, "visible":True}
-        self.label_driver_detail = Gtk.Label(label=_("Searching for available "
-                                                     "drivers..."), **props)
-        self.box_driver_detail.add(self.label_driver_detail)
-
         self.progress_bar = Gtk.ProgressBar()
         self.box_driver_action.pack_end(self.progress_bar, False, False, 0)
         self.progress_bar.set_visible(False)
 
         self.devices = {}
-        self.detect_called = False
         self.driver_changes = []
         self.orig_selection = {}
         # HACK: the case where the selection is actually "Do not use"; is a little
@@ -1370,22 +1344,66 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.nonfree_drivers = 0
         self.ui_building = False
 
+    def on_detect_drivers_cancel_retry(self, button):
+        if self.search_process.is_alive():
+            self.search_process.terminate()
+        else:
+            self.maybe_detect_drivers()
+        self.update_drivers_control_button()
+
+    def update_drivers_control_button(self):
+        if self.search_process.is_alive():
+            self.button_drivers_control.set_label(_("Cancel"))
+        else:
+            self.button_drivers_control.set_label(_("Retry"))
+
+    def wrapper_system_device_drivers(self, queue, cache):
+        devices = detect.system_device_drivers(cache)
+        queue.put_nowait(devices)
+
     def detect_drivers(self):
         # WARNING: This is run in a separate thread.
-        self.detect_called = True
+        queue = multiprocessing.Queue()
+        self.search_process = Process(
+            target=self.wrapper_system_device_drivers,
+            args=(queue, self.apt_cache,),
+        )
+        self.search_process.start()
+        GLib.idle_add(lambda: self.update_drivers_control_button())
+        error = None
         try:
-            self.devices = detect.system_device_drivers(self.apt_cache)
-        except:
+            self.search_process.join()
+            if self.search_process.exception:
+                raise(self.search_process.exception)
+            if self.search_process.exitcode == 0:
+                self.devices = queue.get_nowait()
+            # In the multiprocessing module, if it was terminated by signal N,
+            # the exit code will be the negative value -N.
+            elif self.search_process.exitcode == -SIGTERM:
+                error = _("The search for drivers was cancelled.")
+            else:
+                raise Exception(f"Exit code: {self.search_process.exitcode}")
+        except Exception as e:
             # Catch all exceptions and feed them to apport.
-            GLib.idle_add(self.label_driver_detail.set_text,
-                          _("An error occurred while searching for drivers."))
+            error = _("An error occurred while searching for drivers.")
+            logging.error("An error occurred while searching for drivers."
+                          "Exception: %s", e)
 
             # For apport to catch this exception. See
             # http://bugs.python.org/issue1230540
             sys.excepthook(*sys.exc_info())
+
+        # Using an Idle to ensure the function is called in the main (UI) thread
+        GLib.idle_add(lambda: self.on_detect_drivers_done(error))
+
+    def on_detect_drivers_done(self, err=None):
+        if err is None:
+            self.button_drivers_control.set_sensitive(False)
+            self.show_drivers()
             return
 
-        GLib.idle_add(self.show_drivers)
+        self.label_driver_detail.set_text(err)
+        self.update_drivers_control_button()
 
     def on_driver_selection_changed(self, button, modalias, pkg_name=None):
         if self.ui_building:
@@ -1542,6 +1560,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         return (overall_status, icon, returned_drivers)
 
     def show_drivers(self):
+        self.box_driver_detail.remove(self.buttonbox_driver_detail)
         if not self.devices:
             # No drivers found.
             self.label_driver_detail.set_text(_("No additional drivers available."))
