@@ -34,10 +34,10 @@ gi.require_version('PackageKitGlib', '1.0')
 from gi.repository import PackageKitGlib as packagekit
 from gi.repository import Gio
 
-from PyQt5 import uic
-from PyQt5.QtCore import *
-from PyQt5.QtGui import *
-from PyQt5.QtWidgets import *
+from PyQt6 import uic
+from PyQt6.QtCore import *
+from PyQt6.QtGui import *
+from PyQt6.QtWidgets import *
 
 import softwareproperties
 import softwareproperties.distro
@@ -74,6 +74,13 @@ class SoftwarePropertiesQtUI(QWidget):
   def __init__(self, datadir):
       QWidget.__init__(self)
       uic.loadUi("%s/designer/main.ui" % datadir, self)
+      self.center_window()
+
+  def center_window(self):
+      screen_geometry = QApplication.primaryScreen().availableGeometry()
+      window_geometry = self.frameGeometry()
+      window_geometry.moveCenter(screen_geometry.center())
+      self.move(window_geometry.topLeft())
 
 class SoftwarePropertiesQt(SoftwareProperties):
   def __init__(self, datadir=None, options=None, parent=None, file=None, attachWinID=None):
@@ -108,7 +115,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
     # rejected() signal from Close button
     self.userinterface.buttonBox.rejected.connect(self.on_close_button)
 
-    self.userinterface.buttonBox.button(QDialogButtonBox.Reset).setEnabled(False)
+    self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setEnabled(False)
 
     # Put some life into the user interface:
     self.init_server_chooser()
@@ -146,9 +153,9 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.userinterface.treeview_cdroms.itemChanged.connect(self.on_cdrom_source_toggled)
     self.userinterface.treeview2.itemClicked.connect(self.on_treeview_keys_cursor_changed)
 
-    self.button_close = self.userinterface.buttonBox.button(QDialogButtonBox.Close)
+    self.button_close = self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Close)
     self.button_close.setIcon(QIcon.fromTheme("dialog-close"))
-    self.button_revert = self.userinterface.buttonBox.button(QDialogButtonBox.Reset)
+    self.button_revert = self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Reset)
     self.button_revert.setIcon(QIcon.fromTheme("edit-undo"))
     self.button_revert.clicked.connect(self.on_button_revert_clicked)
 
@@ -356,9 +363,9 @@ class SoftwarePropertiesQt(SoftwareProperties):
             if inconsistent:
                 child.setCheckState(Qt.PartiallyChecked)
             elif active:
-                child.setCheckState(Qt.Checked)
+                child.setCheckState(Qt.CheckState.Checked)
             else:
-                child.setCheckState(Qt.Unchecked)
+                child.setCheckState(Qt.CheckState.Unchecked)
 
     # Enable or disable the component checkbuttons
     for child in self.userinterface.vbox_dist_comps_frame.children():
@@ -368,9 +375,9 @@ class SoftwarePropertiesQt(SoftwareProperties):
             if inconsistent:
                 child.setCheckState(Qt.PartiallyChecked)
             elif active:
-                child.setCheckState(Qt.Checked)
+                child.setCheckState(Qt.CheckState.Checked)
             else:
-                child.setCheckState(Qt.Unchecked)
+                child.setCheckState(Qt.CheckState.Unchecked)
 
     # If no components are enabled there will be no need for updates
     # and source code
@@ -384,11 +391,11 @@ class SoftwarePropertiesQt(SoftwareProperties):
     # Check for source code sources
     source_code_state = self.get_source_code_state()
     if source_code_state == True:
-        self.userinterface.checkbutton_source_code.setCheckState(Qt.Checked)
+        self.userinterface.checkbutton_source_code.setCheckState(Qt.CheckState.Checked)
     elif source_code_state == None:
         self.userinterface.checkbutton_source_code.setCheckState(Qt.PartiallyChecked)
     else:
-        self.userinterface.checkbutton_source_code.setCheckState(Qt.Unchecked)
+        self.userinterface.checkbutton_source_code.setCheckState(Qt.CheckState.Unchecked)
 
     # Will show a short explanation if no CDROMs are used
     if len(self.get_cdrom_sources()) == 0:
@@ -474,9 +481,9 @@ class SoftwarePropertiesQt(SoftwareProperties):
     """
     # FIXME: I find it rather questionable whether sender will work with pyqt doing weird signal handling
     tickBox = kapp.sender()
-    if tickBox.checkState() == Qt.Checked:
+    if tickBox.checkState() == Qt.CheckState.Checked:
         self.enable_component(str(tickBox.objectName()))
-    elif tickBox.checkState() == Qt.Unchecked:
+    elif tickBox.checkState() == Qt.CheckState.Unchecked:
         self.disable_component(str(tickBox.objectName()))
     self.set_modified_sourceslist()
 
@@ -490,18 +497,18 @@ class SoftwarePropertiesQt(SoftwareProperties):
     tickBox = kapp.sender()
     name = str(tickBox.objectName())
 
-    if tickBox.checkState() == Qt.Checked:
+    if tickBox.checkState() == Qt.CheckState.Checked:
         self.enable_child_source(name)
-    elif tickBox.checkState() == Qt.Unchecked:
+    elif tickBox.checkState() == Qt.CheckState.Unchecked:
         self.disable_child_source(name)
     # no way to set back to mixed state so turn off tristate
     tickBox.setTristate(False)
 
   def on_checkbutton_source_code_toggled(self):
     """ Disable or enable the source code for all sources """
-    if self.userinterface.checkbutton_source_code.checkState() == Qt.Checked:
+    if self.userinterface.checkbutton_source_code.checkState() == Qt.CheckState.Checked:
         self.enable_source_code_sources()
-    elif self.userinterface.checkbutton_source_code.checkState() == Qt.Unchecked:
+    elif self.userinterface.checkbutton_source_code.checkState() == Qt.CheckState.Unchecked:
         self.disable_source_code_sources()
     self.userinterface.checkbutton_source_code.setTristate(False)
 
@@ -576,13 +583,13 @@ class SoftwarePropertiesQt(SoftwareProperties):
     SoftwareProperties.revert(self)
     self.set_modified_sourceslist()
     self.show_auto_update_level()
-    self.userinterface.buttonBox.button(QDialogButtonBox.Reset).setEnabled(False)
+    self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setEnabled(False)
     self.modified_sourceslist = False
 
   def set_modified_config(self):
     """The config was changed and now needs to be saved and reloaded"""
     SoftwareProperties.set_modified_config(self)
-    self.userinterface.buttonBox.button(QDialogButtonBox.Reset).setEnabled(True)
+    self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setEnabled(True)
 
   def set_modified_sourceslist(self):
     """The sources list was changed and now needs to be saved and reloaded"""
@@ -590,7 +597,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.show_distro()
     self.show_isv_sources()
     self.show_cdrom_sources()
-    self.userinterface.buttonBox.button(QDialogButtonBox.Reset).setEnabled(True)
+    self.userinterface.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setEnabled(True)
 
   def show_isv_sources(self):
     """ Show the repositories of independent software vendors in the
@@ -620,9 +627,9 @@ class SoftwarePropertiesQt(SoftwareProperties):
         contents = contents.replace("</b>", "")
         item = QTreeWidgetItem([contents])
         if not source.disabled:
-            item.setCheckState(0, Qt.Checked)
+            item.setCheckState(0, Qt.CheckState.Checked)
         else:
-            item.setCheckState(0, Qt.Unchecked)
+            item.setCheckState(0, Qt.CheckState.Unchecked)
         items.append(item)
     treeview.addTopLevelItems(items)
 
@@ -646,7 +653,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
   def on_auto_update_toggled(self):
     """Enable or disable automatic updates and modify the user interface
        accordingly"""
-    if self.userinterface.checkbutton_auto_update.checkState() == Qt.Checked:
+    if self.userinterface.checkbutton_auto_update.checkState() == Qt.CheckState.Checked:
       self.userinterface.combobox_update_interval.setEnabled(True)
       self.userinterface.radiobutton_updates_inst_sec.setEnabled(True)
       self.userinterface.radiobutton_updates_download.setEnabled(True)
@@ -685,7 +692,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
         else:
             dialogue = DialogEdit(self.userinterface, self.sourceslist, self.isv_sources[index], self.datadir)
         result = dialogue.run()
-        if result == QDialog.Accepted:
+        if result == QDialog.DialogCode.Accepted:
             self.replace_source_entry(str(self.isv_sources[index]),
                                       str(dialogue.new_source_entry))
             self.set_modified_sourceslist()
@@ -732,7 +739,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
     if item == None:
         return
     key = item.text(0)
-    if not self.remove_key(key[:16]):
+    if not self.remove_key(key.split("\n", 1)[0]):
       title = _("Error removing the key")
       text = _("The key you selected could not be removed. "
                "Please report this as a bug.")
@@ -764,14 +771,14 @@ class SoftwarePropertiesQt(SoftwareProperties):
        only if there is no parent defined"""
     if self.modified_sourceslist == True and self.options.no_update == False:
         messageBox = QMessageBox(self.userinterface)
-        messageBox.setIcon(QMessageBox.Information)
-        reloadButton = messageBox.addButton(_("Reload"), QMessageBox.AcceptRole)
-        messageBox.addButton(QMessageBox.Close)
+        messageBox.setIcon(QMessageBox.Icon.Information)
+        reloadButton = messageBox.addButton(_("Reload"), QMessageBox.ButtonRole.AcceptRole)
+        messageBox.addButton(QMessageBox.StandardButton.Close)
         primary = _("Your local copy of the software catalog is out of date.")
         secondary = _("A new copy will be downloaded.")
         text = "%s<br /><br /><small>%s</small>" % (primary, secondary)
         messageBox.setText(text)
-        messageBox.exec_()
+        messageBox.exec()
         if (messageBox.clickedButton() == reloadButton):
                 self._pdialog = QProgressDialog("<big>{}</big>".format(_("Refreshing software cache")),
                                                 "Cancel", 0, 100, self.userinterface)
@@ -840,7 +847,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
       self.set_modified_sourceslist()
 
   def run(self):
-    kapp.exec_()
+    kapp.exec()
 
   def on_driver_changes_progress(self, progress, ptype, data=None):
     self.button_driver_revert.setVisible(False)
@@ -971,8 +978,6 @@ class SoftwarePropertiesQt(SoftwareProperties):
     elif (desktop == 'XFCE' and
       os.path.exists('/usr/bin/xfce4-session-logout')):
       subprocess.call(['xfce4-session-logout'])
-    elif (desktop == 'LXDE' and os.path.exists('/usr/bin/lubuntu-logout')):
-      subprocess.call(['lubuntu-logout'])
     elif (desktop == 'LXQt' and os.path.exists('/usr/bin/lxqt-leave')):
       subprocess.call(['lxqt-leave'])
 
@@ -1007,7 +1012,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.userinterface.box_driver_action.addWidget(self.button_driver_cancel)
 
     self.label_driver_detail = QLabel("Searching for available drivers...")
-    self.label_driver_detail.setAlignment(Qt.AlignCenter)
+    self.label_driver_detail.setAlignment(Qt.AlignmentFlag.AlignCenter)
     self.userinterface.box_driver_detail.addWidget(self.label_driver_detail)
 
     self.progress_bar = QProgressBar()
@@ -1216,8 +1221,8 @@ licence=licence)
       (overall_status, icon, drivers) = self.gather_device_data(self.devices[device])
 
       driver_status = QLabel()
-      driver_status.setAlignment(Qt.AlignTop)
-      driver_status.setAlignment(Qt.AlignHCenter)
+      driver_status.setAlignment(Qt.AlignmentFlag.AlignTop)
+      driver_status.setAlignment(Qt.AlignmentFlag.AlignHCenter)
       pixmap = QIcon.fromTheme(icon).pixmap(QSize(16, 16))
       driver_status.setPixmap(pixmap)
 
@@ -1227,10 +1232,10 @@ licence=licence)
       device_box.addLayout(device_detail,1)#1 for priority over the icon to stretch
 
       widget = QLabel("{}: {}".format(self.devices[device].get('vendor', _('Unknown')), self.devices[device].get('model', _('Unknown'))))
-      widget.setAlignment(Qt.AlignLeft)
+      widget.setAlignment(Qt.AlignmentFlag.AlignLeft)
       device_detail.addWidget(widget)
       widget = QLabel("<small>{}</small>".format(overall_status))
-      widget.setAlignment(Qt.AlignLeft)
+      widget.setAlignment(Qt.AlignmentFlag.AlignLeft)
       #widget.set_use_markup(True)
       device_detail.addWidget(widget)
       self.dynamic_device_status[device] = (driver_status, widget)
